
import java.io.*;
import java.net.*;
/**
 * An Echo server
 *
 * @author Steven Ring
 */

public class EchoServer {

	/** port on which to listen for connections */
	private int port = 0;

	/** count of clients served */
	private int clientId = 0;

	/**
	 * Constructor
	 */
	public EchoServer( int port ) {
		this.port = port;
	}

	/**
	 * open a socket and start listening on it
	 */
	public void listen() {

		ServerSocket ss = null;

		// get the socket on which to listen

		try {
			ss =  new ServerSocket( port );
		} catch ( IOException e ) {
			usage( "Cannot open port " + port + ", " + e.getMessage() );
		}

		// start listening for connections


		try {
			while ( true ) {
			
				Socket s = ss.accept();
			
				Server server = new Server( s, clientId );
				clientId++;
				server.start();
			}
		} catch ( IOException e ) { 
			usage( "Error while accepting connection, " + e.getMessage() );
		}

	}


	public static void main( String [] arg ) {

		int port = 0;

		// check parameter

		if ( arg.length < 1 ) 
			usage( "Missing port parameter" );

		// parse out the port parameter

		try {
			port = Integer.parseInt( arg[0] );
		} catch ( Exception e ) {
			usage( "Cannot parse integer, " + e.getMessage() );
		}

		// instantiate the server and start it
	
		EchoServer myServer = new EchoServer( port );

		// and start listening for connections

		myServer.listen();
		
	}

	/**
 	 * How to use the program
	 */
	private static void usage( String message ) {

		if ( null != message ) 
			System.out.println( message );
		
		System.out.println( "\nUsage:  java EchoServer <port>\n" );
		System.exit(1);
	}


	/**
	 * a class to service a single client
	 *
	 */
	class Server extends Thread {

		/** an Id for this server */
		int id = 0; 

		/** the socket on which to communicate with the client */
		Socket socket;

		/**
		 * construct a new server
		 */
		public Server( Socket socket, int id ) {

			this.socket = socket; 
			this.id = id;
		}

		/**
		 * The main process for this server
		 */
		public void run() {

			try {

				// get a reader to read the client's data

				BufferedReader in = new BufferedReader( 
									new InputStreamReader(
										socket.getInputStream() ) );

				// get a writer to send response to client

				PrintWriter out = new PrintWriter( 
										socket.getOutputStream(), true );


				// welcome the user

				out.println( "Welcome to the echo server" );
				out.println( "You are client " + id );
				out.println( "Type '-exit' to logout" );

				// write console message

				System.out.println( "Client " + id + ": logged on" );

				// start the main echo loop

				boolean done = false;

				while ( !done ) {
					String line = in.readLine();
					if ( null == line ) {
						done = true;
						System.out.println( "Client " + id + ": connection lost" );
					} else if ( line.equals( "-exit" ) ) {
						done = true; 
						out.println( "I'll be going now" );
					} else {

						// put client's message on server console

						System.out.println( "Client " + id + ": " + line );

						// echo to client

						out.println( line );
					}
				}
			} catch ( IOException e ) {
				System.out.println( "Server run() method got exception: " +
							e.getMessage() );
			} finally {
				
				// all done so tidy up

				try {
					socket.close();
				} catch ( Exception e ) {
				}
				System.out.println( "Client " + id + ": logged off" );
			}
		}
	}

}
