import java.io.*;
import java.net.*;
/**
 * A receiver  of UDP packets
 *
 * @author Steven Ring
 */

public class Destination {

	private static final int MAX_PACKET_SIZE = 4096;


	/** port on which to listen for connections */
	private int port = 0;

	/**
	 * Constructor
	 */
	public Destination( int port ) {
		this.port = port;
	}


	/**
	 * Send some data to all who want to listen
	 */

	public void receiveSomething() {

		// buffer for data

		byte [] data = new byte[ MAX_PACKET_SIZE ];
		DatagramSocket s = null;

		try {
			s = new DatagramSocket( port );
		} catch ( Exception e ) {
			usage( "Error opening receiving socket: " + e.getMessage() );
		}

		// loop reading datagrams

		while( true ) {
			try {
				DatagramPacket p = new DatagramPacket( data, data.length );
				s.receive(p);

				// now decode the data in the byte array

				ByteArrayInputStream bais = new ByteArrayInputStream( data );
				DataInputStream in = new DataInputStream( bais );
	
				int i1 = in.readInt();
				int i2 = in.readInt();
				int i3 = in.readInt();
				int i4 = in.readInt();

				// output result to console

				System.out.println( "Received datagram with data "
									+i1+" "+i2+" "+i3+" "+i4 );

			} catch ( Exception e ) {
				usage( "Error reading datagram: " + e.getMessage() );
			}
		}
	}

	public static void main( String [] arg ) {

		String ipAddress = null;
		int port = 0;

		// check parameter

		if ( arg.length < 1 ) 
			usage( "Missing <port> parameter" );

		// parse out the port parameter

		try {
			port = Integer.parseInt( arg[0] );
		} catch ( Exception e ) {
			usage( "Cannot parse integer, " + e.getMessage() );
		}

		// instantiate the server and start it
	
		Destination myDestination = new Destination( port );

		// and start listening for connections

		myDestination.receiveSomething();
		
	}

	/**
 	 * How to use the program
	 */
	private static void usage( String message ) {

		if ( null != message ) 
			System.out.println( message );
		
		System.out.println( "\nUsage:  java Destination <port>\n" );
		System.exit(1);
	}
}
