import java.io.*;
import java.net.*;
/**
 * A source of UDP packets
 *
 * @author Steven Ring
 */

public class Source {

	private static final int MAX_PACKET_SIZE = 4096;

	/** the IP address of the destination */
	String ipAddress = null;

	/** port on which to listen for connections */
	private int port = 0;

	/**
	 * Constructor
	 */
	public Source( String ipAddress, int port ) {
		this.ipAddress = ipAddress;
		this.port = port;
	}


	/**
	 * Send some data to all who want to listen
	 */

	public void sendSomething() {

		InetAddress dest = null;
		try {

			dest = InetAddress.getByName( ipAddress );
		} catch ( Exception e ) {
			usage( e.getMessage() );
		}

		ByteArrayOutputStream baos = new ByteArrayOutputStream( MAX_PACKET_SIZE );
		DataOutputStream out = new DataOutputStream( baos );

		// now send a sequenc of 4 integers ... "2 4 6 8"

		byte [] data = null;
		try {
			out.writeInt( 1112 );
			out.writeInt( 1114 );
			out.writeInt( 1116 );
			out.writeInt( 1118 );
			out.flush();
			data = baos.toByteArray();
		} catch ( Exception e ) {
			usage( "Error formatting data: " + e.getMessage() );
		}
		

		// now send the resulting byte array as a datagram

		try {
			DatagramSocket s = new DatagramSocket();
			DatagramPacket p = new DatagramPacket( data, data.length, dest, port);

			s.send(p);
			s.close();
		
		} catch ( Exception e ) {
			usage( "Error sending datagram: " + e.getMessage() );
		}
	}

	public static void main( String [] arg ) {

		String ipAddress = null;
		int port = 0;

		// check parameter

		if ( arg.length < 2 ) 
			usage( "Missing <ip adress> and/or <port> parameter" );

		// parse out the port parameter

		ipAddress = arg[0];
		try {
			port = Integer.parseInt( arg[1] );
		} catch ( Exception e ) {
			usage( "Cannot parse integer, " + e.getMessage() );
		}

		// instantiate the server and start it
	
		Source mySource = new Source( ipAddress, port );

		// and start listening for connections

		mySource.sendSomething();
		
	}

	/**
 	 * How to use the program
	 */
	private static void usage( String message ) {

		if ( null != message ) 
			System.out.println( message );
		
		System.out.println( "\nUsage:  java Source <ip Address> <port>\n" );
		System.exit(1);
	}
}
